/*****************************************************************************/
/*
                                 synopsis.c

Section synopsis and blog synopsis processing.

                                      
COPYRIGHT
---------
Copyright (C) 2008-2011 Mark G.Daniel
This program, comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it under the
conditions of the GNU GENERAL PUBLIC LICENSE, version 3, or any later version.
http://www.gnu.org/licenses/gpl.txt


VERSION HISTORY
---------------
02-MAY-2011  MGD  SynopsisContent() XML item RssEscape() (per TES)
09-APR-2010  MGD  initial (fron CONTENT.C)
*/

/*****************************************************************************/

#ifdef VWCMS_VMS_V62
#undef _VMS_V6_SOURCE
#define _VMS_V6_SOURCE
#undef __VMS_VER
#define __VMS_VER 60200000
#undef __CRTL_VER
#define __CRTL_VER 60200000
#endif

#ifndef __VAX
#   pragma nomember_alignment
#endif

/* standard C header files */
#include <ctype.h>
#include <errno.h>
#include <stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <types.h>
#include <time.h>

/* VMS related header files */
#include "ssdef.h"
#include "stsdef.h"

/* application header file */
#include "ago.h"
#include "cgilib.h"
#include "config.h"
#include "content.h"
#include "manage.h"
#include "other.h"
#include "search.h"
#include "synopsis.h"
#include "tag.h"
#include "template.h"
#include "vwcms.h"

#define FI_LI "SYNOPSIS", __LINE__

/* prototypes */

/* external storage */

extern BOOL  Debug,
             QueryRss,
             WatchEnabled;

extern int  DataCacheMax;

extern ulong  CurrentSecondsTime;

extern char  *CgiRequestUriPtr,
             *LastModifiedNow;

extern char  LinkSchemeHost[],
             SoftwareId[],
             TimeZoneString[];

extern CONTENT_DATA  *ContentSitePtr,
                     *ContentSectionPtr;

extern CONFIG_DATA  VWcmsConfig;

/*****************************************************************************/
/*
Free the linked-list of sections.
*/

void SynopsisFree (CONTENT_DATA *cdptr)

{
   CONTENT_SYNOPSIS  *csptr, *ncsptr;

   /*********/
   /* begin */
   /*********/

   if (Debug) fprintf (stdout, "SynopsisFree()\n");

   for (csptr = cdptr->SynopsisPtr; csptr; csptr = ncsptr)
   {
      ncsptr = csptr->NextPtr;
      free (csptr);
   }
   cdptr->SynopsisPtr = NULL;
   cdptr->SynopsisOrderedAs = 0;
}

/*****************************************************************************/
/*
Create a linked-list of sections.  This list can be sorted against various
criteria.  Keep the list until the content is released by ContentUnParse().
*/

void SynopsisScan ()

{
   ulong  TimeStampSeconds = 0;
   char  *cptr, *sptr, *zptr;
   char  TimeStamp [256];
   struct tm  time_struct;
   CONTENT_DATA  *cdptr;
   CONTENT_SYNOPSIS  *csptr, *ncsptr, *tcsptr,
                     *TailPtr;

   /*********/
   /* begin */
   /*********/

   if (Debug) fprintf (stdout, "SynopsisScan()\n");

   /* if it's already been scanned */
   if (ContentSitePtr->SynopsisOrderedAs) return;

   for (cdptr = ContentSitePtr; cdptr; cdptr = cdptr->NextPtr)
   {
      if (cdptr->IsHidden) continue;

      if (cdptr->BlogPtr)
         cptr = cdptr->BlogPtr;
      else
         cptr = NULL;

      if (cptr)
      {
         /* look for an indication it's a time string (i.e. HH:MM) */
         for (sptr = cptr;
              *sptr && *sptr != '$' && *sptr != '=' && *sptr != '.';
              sptr++);
         /* if must another parameter */
         if (*sptr != '$' && *sptr != '=' && *sptr != '.') cptr = NULL;
      }

      if (cptr)
      {
         zptr = (sptr = TimeStamp) + sizeof(TimeStamp)-1;
         while (*cptr && *cptr != '$' && sptr < zptr) *sptr++ = *cptr++;
         *sptr = '\0';

         sptr = GetTimeString (TimeStamp, &time_struct);
         if (sptr) TimeStampSeconds = mktime (&time_struct);
         if ((int)TimeStampSeconds == -1) sptr = NULL;
         /* if a problem converting the date/time */
         if (!sptr) continue;

         /* parse to the end of the directive */
         while (*cptr && *(USHORTPTR)cptr != '$$') cptr++;
         if (*cptr) cptr += 2;
      }
      else
         cptr = cdptr->TextPtr;

      while (*cptr && isspace(*cptr)) cptr++;
      /* if appears to have no content */
      if (!*cptr) continue;

      csptr = (CONTENT_SYNOPSIS*) calloc (1, sizeof(CONTENT_SYNOPSIS));
      if (!csptr) ErrorExit (vaxc$errno, FI_LI);

      csptr->ContentPtr = cdptr;
      csptr->TimeStampSeconds = TimeStampSeconds;
      csptr->SectionNumber[0] = cdptr->SectionNumber[0];
      csptr->SectionNumber[1] = cdptr->SectionNumber[1];
      csptr->SectionNumber[2] = cdptr->SectionNumber[2];
      csptr->SectionNumber[3] = cdptr->SectionNumber[3];
      csptr->SectionNumber[4] = cdptr->SectionNumber[4];
      csptr->SectionNumber[5] = cdptr->SectionNumber[5];
      csptr->SectionLastUpdated = cdptr->SectionLastUpdated;
      csptr->IsBlogEntry = (cdptr->BlogPtr != NULL);
      csptr->ExtractPtr = cptr;

      if (!ContentSitePtr->SynopsisPtr)
         ContentSitePtr->SynopsisPtr = TailPtr = csptr;
      else
      {
         TailPtr->NextPtr = csptr;
         csptr->PrevPtr = TailPtr;
         TailPtr = csptr;
      }
   }

   ContentSitePtr->SynopsisOrderedAs = CONTENT_ORDERED_FIRST_TO_LAST;
}

/*****************************************************************************/
/*
(Bubble) sort synposis list into the parameter order.
*/

void SynopsisSort (int OrderAs)

{
   BOOL SwapEm;
   CONTENT_SYNOPSIS  *csptr, *ncsptr, *tcsptr;

   /*********/
   /* begin */
   /*********/

   if (Debug) fprintf (stdout, "SynopsisSort()\n");

   if (WatchEnabled)
      for (csptr = ContentSitePtr->SynopsisPtr; csptr; csptr = csptr->NextPtr)
         WatchThis (FI_LI, "SORTED <H!UL> !AZ !AZ",
                    csptr->ContentPtr->HeadNumber,
                    csptr->ContentPtr->SectionNumberDot,
                    csptr->ContentPtr->SectionName);

   if (OrderAs == ContentSitePtr->SynopsisOrderedAs) return; 

   ContentSitePtr->SynopsisOrderedAs = OrderAs;

   for (csptr = ContentSitePtr->SynopsisPtr; csptr; csptr = csptr->NextPtr)
   {
      for (ncsptr = csptr->NextPtr; ncsptr; ncsptr = ncsptr->NextPtr)
      {
         SwapEm = FALSE;
         switch (OrderAs)
         {
            case CONTENT_ORDERED_FIRST_TO_LAST :
               if (ncsptr->SectionNumber[0] < csptr->SectionNumber[0])
                  SwapEm = TRUE;
               break;
            case CONTENT_ORDERED_LAST_TO_FIRST :
               if (ncsptr->SectionNumber[0] > csptr->SectionNumber[0])
                  SwapEm = TRUE;
               break;
            case CONTENT_ORDERED_BLOG_MOST_RECENT :
               if (ncsptr->TimeStampSeconds > csptr->TimeStampSeconds)
                  SwapEm = TRUE;
               break;
            case CONTENT_ORDERED_BLOG_LEAST_RECENT :
               if (ncsptr->TimeStampSeconds < csptr->TimeStampSeconds)
                  SwapEm = TRUE;
               break;
            case CONTENT_ORDERED_UPDATED_MOST_RECENT :
               if (ncsptr->SectionLastUpdated > csptr->SectionLastUpdated)
                  SwapEm = TRUE;
               break;
            case CONTENT_ORDERED_UPDATED_LEAST_RECENT :
               if (ncsptr->SectionLastUpdated < csptr->SectionLastUpdated)
                  SwapEm = TRUE;
               break;
            default :
               ErrorExit (SS$_BUGCHECK, FI_LI);
         }

         if (SwapEm)
         {
            if (ContentSitePtr->SynopsisPtr == csptr)
               ContentSitePtr->SynopsisPtr = ncsptr;

            tcsptr = ncsptr->PrevPtr;
            ncsptr->PrevPtr = csptr->PrevPtr;
            csptr->PrevPtr = tcsptr;
            if (csptr->PrevPtr) csptr->PrevPtr->NextPtr = csptr;
            if (ncsptr->PrevPtr) ncsptr->PrevPtr->NextPtr = ncsptr;

            tcsptr = ncsptr->NextPtr;
            ncsptr->NextPtr = csptr->NextPtr;
            csptr->NextPtr = tcsptr;
            if (csptr->NextPtr) csptr->NextPtr->PrevPtr = csptr;
            if (ncsptr->NextPtr) ncsptr->NextPtr->PrevPtr = ncsptr;

            tcsptr = ncsptr;
            ncsptr = csptr;
            csptr = tcsptr;
         }
      }
   }

   if (WatchEnabled)
      for (csptr = ContentSitePtr->SynopsisPtr; csptr; csptr = csptr->NextPtr)
         WatchThis (FI_LI, "SORTED <H!UL> !AZ !AZ",
                    csptr->ContentPtr->HeadNumber,
                    csptr->ContentPtr->SectionNumberDot,
                    csptr->ContentPtr->SectionName);
}

/*****************************************************************************/
/*
A VWcms synopsis is a listing of site sections (pages) showing the title and an
extract from the section content.  A VWcms blog is a specific subgroup of the
more general synopsis.  RSS feeds (both type=blog and type=synopsis) are also
generated by this function.

There are four levels of synopsis.

'1' for the entire site.  Usually used inconjunction with with "since=.." to
list sections that have been updated since in the specified period.

'0' that lists everything at the directive level used.  So an <H1> will
synopsis all <H1>s.  An <H2> will synposis all <H2>s until the next <H1> is
encountered.  Like wise, an <H3> will synopsis all <H3>s uintil the next <H2>
is encountered.

'-1' lists all subordinate levels until the next peer level is encountered. 
So an <H1> will synposis all <H2> until the next <H1>, a <H2> all <H3>s until
the next <H2>, while an <H3> will synopsis nothing.

'-2' will synposis the subordinate or all subordinates.  In other words it only
applies to <H1>s.  A '-2' will list all <H3>s until the next <H1> is
encountered.

The first parameter may also be a '.' (end-of-synopsis), detected during this
function's processing.  It terminates extract text at that point.
*/

void SynopsisContent
(
char *SynopsisSubsArg,
char *BlogSubsArg
)
{
   BOOL  HitTag,
         NotList = TRUE;
   int  cnt,
        AmpCount,
        ExtractLength = 0,
        ExtractSize = 0,
        HeadNumber,
        ItemCount = 0,
        ListLength = 999999999,
        RssTTL = 0,
        SortOrder,
        SynopsisLevel;
   int  SectionNumber [6];
   ulong  SinceSeconds = 0;
   char  *cptr, *sptr, *tptr, *zptr,
         *CloudPtr = NULL,
         *ExtractPtr= NULL,
         *RssDescriptionPtr,
         *RssTitlePtr,
         *ReadPtr,
         *SectionNamePtr,
         *TagPtr = NULL,
         *NarrowClass = "",
         *TitlePtr;
   char  RssDescription [256] = "",
         RssLanguage [256] = { 'e','n','-','u','s',0 },
         RssEditor [256] = "",
         RssTitle [256] = "",
         RssWebMaster [256] = "",
         SynopsisEmpty [256] = "",
         SynopsisGoto [256] = "",
         SynopsisRead [256] = "",
         SynopsisTitle [256] = "",
         TagWith [256] = "";
   struct tm  time_struct;
   CONTENT_DATA  *cdptr;
   CONTENT_SYNOPSIS  *csptr;

   /*********/
   /* begin */
   /*********/

   if (Debug) fprintf (stdout, "SynopsisContent()\n");

   if (!ContentSitePtr) return;

   if (WatchEnabled)
      WatchThis (FI_LI, "SYNOPSIS !AZ !AZ",
                 SynopsisSubsArg ? "synopsis" : "blog",
                 SynopsisSubsArg ? SynopsisSubsArg : BlogSubsArg);

   if (cptr = SynopsisSubsArg)
   {
      SortOrder = 0;
      SynopsisLevel = -1;
      ExtractLength = SYNOPSIS_EXTRACT_DEFAULT;
   }
   else
   if (cptr = BlogSubsArg)
   {
      SortOrder = -1;
      SynopsisLevel = 1;
      ExtractLength = BLOG_RECENT_EXTRACT_DEFAULT;
   }
   else
      ErrorExit (SS$_BUGCHECK, FI_LI);

   if (*cptr == '$') cptr++;

   if (strsame (cptr, "LATEST=", 7))
   {
      /* shouldn't get here unless an applicable blog was not found */
      fprintf (stdout, "<div class=\"vwcms_no_content\">%s</div>\n",
               CONFIG_LANG(LangEmpty));
      TemplateAdvise ("W$$BLOG$!AZ", cptr);
      return;
   }

   while (*cptr)
   {
      /* just an end-of-synopsis directive */
      if (*cptr == '.') return;

      if (strsame (cptr, "cloud=", 6))
      {
         if (!(CloudPtr = GetCgiVarNull ("FORM_CLOUD"))) return;
         SynopsisLevel = 1;
         if (isdigit(cptr[6])) SynopsisLevel = atoi(cptr += 6);
         cptr = TemplateParseSkipParam (cptr, NULL);
      }
      else
      if (strsame (cptr, "description=", 12))
      {
         /* RSS-specific parameter */
         cptr = TemplateParseParam (cptr+12, RssDescription,
                                             sizeof(RssDescription));
      }
      else
      if (strsame (cptr, "editor=", 7))
      {
         /* RSS-specific parameter */
         cptr = TemplateParseParam (cptr+7, RssEditor, sizeof(RssEditor));
      }
      else
      if (strsame (cptr, "empty=", 6))
         cptr = TemplateParseParam (cptr+6, SynopsisEmpty,
                                            sizeof(SynopsisEmpty));
      else
      if (strsame (cptr, "extract=", 8))
      {
         ExtractLength = atoi(cptr += 8);
         cptr = TemplateParseSkipParam (cptr, NULL);
      }
      else
      if (strsame (cptr, "language=", 9))
      {
         /* RSS-specific parameter */
         cptr = TemplateParseParam (cptr+9, RssLanguage, sizeof(RssLanguage));
      }
      else
      if (strsame (cptr, "level=", 6))
      {
         SynopsisLevel = atoi(cptr += 6);
         cptr = TemplateParseSkipParam (cptr, NULL);
      }
      else
      if (strsame (cptr, "list=", 5))
      {
         ListLength = atoi(cptr += 5);
         cptr = TemplateParseSkipParam (cptr, NULL);
      }
      else
      if (strsame (cptr, "narrow", -1))
      {
         NarrowClass = "_narrow";
         cptr = TemplateParseSkipParam (cptr, NULL);
      }
      else
      if (strsame (cptr, "order=+", 7))
      {
         SortOrder = +1;
         cptr = TemplateParseSkipParam (cptr, NULL);
      }
      else
      if (strsame (cptr, "order=-", 7))
      {
         SortOrder = -1;
         cptr = TemplateParseSkipParam (cptr, NULL);
      }
      else
      if (strsame (cptr, "read=", 5))
         cptr = TemplateParseParam (cptr+5, SynopsisRead, sizeof(SynopsisRead));
      else
      if (strsame (cptr, "since=", 6))
      {
         cptr += 6;
         if (*cptr == '*')
         {
            /* since 'forever' */
            SinceSeconds = 9999 * 86400;
         }
         else
         if (*cptr == '-')
         {
            /* number of days before current */
            SinceSeconds = CurrentSecondsTime + (atoi(cptr) * 86400);
         }
         else
         {
            /* since specified date/time string */
            sptr = GetTimeString (cptr, &time_struct);
            if (sptr)
            {
               SinceSeconds = mktime (&time_struct);
               if ((int)SinceSeconds == -1) sptr = NULL;
            }
            if (!sptr)
            {
               TemplateAdvise ("E!AZ", HtmlEscape(cptr));
               return;
            }
         }
         cptr = TemplateParseSkipParam (cptr, NULL);
      }
      else
      if (strsame (cptr, "subscribe[=]", 9))
      {
         /* just absorb this $$RSS$$ parameter */
         cptr = TemplateParseSkipParam (cptr+9, NULL);
      }
      else
      if (strsame (cptr, "tag=", 4))
      {
         if (!(TagPtr = GetCgiVarNull ("FORM_TAG"))) return;
         SynopsisLevel = 1;
         if (isdigit(cptr[4])) SynopsisLevel = atoi(cptr += 4);
         cptr = TemplateParseSkipParam (cptr, NULL);
      }
      else
      if (*cptr == '=' || strsame (cptr, "text=", 5) && NotList)
      {
         /*****************/
         /* synopsis text */
         /*****************/

         if (*cptr++ != '=') cptr += 4;
         fputs ("<span class=\"vwcms_synopsis_text\">", stdout);
         /* do it here, do it now; saves scanning to the end twice! */
         for (sptr = cptr; *cptr && *(USHORTPTR)cptr != '$$'; cptr++);
         OutputFromTo (sptr, cptr);
         fputs ("</span>", stdout);
         return;
      }
      else
      if (strsame (cptr, "title=", 6))
      {
         /* RSS-specific parameter */
         cptr = TemplateParseParam (cptr+6, RssTitle, sizeof(RssTitle));
      }
      else
      if (strsame (cptr, "ttl=", 4))
      {
         /* RSS-specific parameter */
         RssTTL = atoi(cptr += 4);
         cptr = TemplateParseSkipParam (cptr, NULL);
      }
      else
      if (strsame (cptr, "webmaster=", 10))
      {
         /* RSS-specific parameter */
         cptr = TemplateParseParam (cptr+10, RssWebMaster,
                                             sizeof(RssWebMaster));
      }
      else
      {
         if (QueryRss)
         {
            CgiLibResponseHeader (200, "text/plain",
                                  "Last-Modified: %s\n", LastModifiedNow);
            fprintf (stdout, "ERROR 500 (%s): $$RSS$%s$$\n",
                     CgiLibHttpStatusCodeText(500), cptr);
         }
         else
         if (SynopsisSubsArg)
            TemplateParamError ("SYNOPSIS", SynopsisSubsArg, cptr);
         else
            TemplateParamError ("BLOG", BlogSubsArg, cptr);
         return;
      }
      NotList = FALSE;
   }

   /*************/
   /* synopsise */
   /*************/

   SynopsisScan ();

   if (SynopsisSubsArg)
   {
      if (!*(ReadPtr = SynopsisRead)) ReadPtr = CONFIG_LANG(LangSynopsis);

      if (ExtractLength != 0)
      {
         if (ExtractLength < SYNOPSIS_EXTRACT_MIN)
            ExtractLength = SYNOPSIS_EXTRACT_DEFAULT;
         else
         if (ExtractLength > SYNOPSIS_EXTRACT_MAX)
            ExtractLength = SYNOPSIS_EXTRACT_MAX;
      }

      if (WatchEnabled)
         WatchThis (FI_LI, "SYNOPSIS <H!UL> !SL !SL",
                    ContentSectionPtr->HeadNumber,
                    SynopsisLevel, ExtractLength);

      if (SortOrder)
      {
         if (SortOrder < 0)
            SynopsisSort (CONTENT_ORDERED_UPDATED_MOST_RECENT);
         else 
            SynopsisSort (CONTENT_ORDERED_UPDATED_LEAST_RECENT);
      }
      else
         SynopsisSort (CONTENT_ORDERED_FIRST_TO_LAST);
   }
   else
   {
      if (!*(ReadPtr = SynopsisRead)) ReadPtr = CONFIG_LANG_PARSE(LangBlog,1);

      if (ListLength < 1)
         ListLength = BLOG_RECENT_LIST_DEFAULT;
      else
      if (ListLength > BLOG_RECENT_LIST_MAX)
         ListLength = BLOG_RECENT_LIST_MAX;

      if (ExtractLength <= 0)
         ExtractLength = 0;
      else
      if (ExtractLength < BLOG_RECENT_EXTRACT_MIN)
         ExtractLength = BLOG_RECENT_EXTRACT_DEFAULT;
      else
      if (ExtractLength > BLOG_RECENT_EXTRACT_MAX)
         ExtractLength = BLOG_RECENT_EXTRACT_MAX;

      if (SortOrder < 0)
         SynopsisSort (CONTENT_ORDERED_BLOG_MOST_RECENT);
      else 
         SynopsisSort (CONTENT_ORDERED_BLOG_LEAST_RECENT);

      csptr = ContentSitePtr->SynopsisPtr;
   }

   HeadNumber = ContentSectionPtr->HeadNumber;

   if (SynopsisLevel > 0)
      memset (SectionNumber, 0, sizeof(SectionNumber));
   else
      memcpy (SectionNumber,
              ContentSectionPtr->SectionNumber,
              sizeof(SectionNumber));

   /**************/
   /* begin list */
   /**************/

   if (QueryRss)
   {
      if (!*(RssTitlePtr = RssTitle))
         RssTitlePtr = ContentSectionPtr->SectionTitle;

      if (!*(RssDescriptionPtr = RssDescription))
         RssDescriptionPtr = ContentSectionPtr->SectionTitle;

      CgiLibResponseHeader (200, "application/rss+xml",
                            "Last-Modified: %s\n", LastModifiedNow);

      fprintf (stdout,
"<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\
<rss version=\"2.0\" xmlns:atom=\"http://www.w3.org/2005/Atom\">\n\
 <channel>\n\
  <generator>%s</generator>\n\
  <atom:link href=\"%s%s\" rel=\"self\" type=\"application/rss+xml\"/>\n\
  <language>%s</language>\n\
  <title>%s</title>\n\
  <link>%s%s</link>\n\
  <description>%s</description>\n",
               SoftwareId,
               LinkSchemeHost,
               CgiRequestUriPtr,
               RssLanguage,
               RssEscape(RssTitlePtr),
               LinkSchemeHost,
               CgiRequestUriPtr,
               RssEscape(RssDescriptionPtr));

      if (RssEditor[0])
         fprintf (stdout, "  <managingEditor>%s</managingEditor>\n",
                  RssEscape(RssEditor));

      if (RssWebMaster[0])
         fprintf (stdout, "  <webMaster>%s</webMaster>\n",
                  RssEscape(RssWebMaster));

      if (RssTTL) fprintf (stdout, "  <ttl>%d</ttl>\n", RssTTL);
   }
   else
   if (SynopsisSubsArg)
      fputs ("<table class=\"vwcms_synopsis_list\" border=\"0\" \
cellpadding=\"0\" cellspacing=\"0\">\n",
             stdout);
   else
      fputs ("<table class=\"vwcms_blog_list\" border=\"0\" \
cellpadding=\"0\" cellspacing=\"0\">\n",
             stdout);

   /********/
   /* list */
   /********/

   if (WatchEnabled)
      WatchThis (FI_LI, "SYNOPSIS !UL !UL.!UL.!UL.!UL.!UL <H!UL>",
                 SectionNumber[0], SectionNumber[1],
                 SectionNumber[2], SectionNumber[3],
                 SectionNumber[4], SectionNumber[5], HeadNumber);

   for (csptr = ContentSitePtr->SynopsisPtr; csptr; csptr = csptr->NextPtr)
   {
      cdptr = csptr->ContentPtr;

      if (WatchEnabled)
         WatchThis (FI_LI,
"SYNOPSIS !UL !UL.!UL.!UL.!UL.!UL.!UL  <H!UL> !AZ !AZ !AZ !AZ",
                    cdptr->SectionNumber[0], cdptr->SectionNumber[1],
                    cdptr->SectionNumber[2], cdptr->SectionNumber[3],
                    cdptr->SectionNumber[4], cdptr->SectionNumber[5],
                    cdptr->SectionNumber[6], cdptr->HeadNumber,
                    cdptr->SectionNumberDot, cdptr->SectionName,
                    cdptr->IsHidden ? "HIDE" : "NOT-hidden",
                    csptr->IsBlogEntry ? "BLOG" : "NOT-blog");

      if (cdptr->IsHidden) continue;
      if (BlogSubsArg && !csptr->IsBlogEntry) continue;
      if (SinceSeconds && SinceSeconds > csptr->SectionLastUpdated) continue;

      if (SynopsisLevel <= 0)
      {
         if (cdptr->SectionNumber[0] <= SectionNumber[0]) continue;
         /* with -1, -2, etc. this subtraction will actually add */
         if (cdptr->HeadNumber != HeadNumber-SynopsisLevel) continue;
         for (cnt = 1; cnt < HeadNumber-SynopsisLevel; cnt++)
            if (cdptr->SectionNumber[cnt] != SectionNumber[cnt]) break;
         if (cnt < HeadNumber-SynopsisLevel) continue;
      }
      if (SynopsisLevel == 0)
      {
         if (cdptr->SectionNumber[0] <= SectionNumber[0]) continue;
         if (cdptr->HeadNumber != HeadNumber) continue;
         for (cnt = 1; cnt < HeadNumber; cnt++)
            if (cdptr->SectionNumber[cnt] != SectionNumber[cnt]) break;
         if (cnt < HeadNumber) continue;
      }
      else
      if (SynopsisLevel == -1)
      {
         if (cdptr->SectionNumber[0] <= SectionNumber[0]) continue;
         if (cdptr->HeadNumber != HeadNumber+1) continue;
         for (cnt = 1; cnt < HeadNumber+1; cnt++)
            if (cdptr->SectionNumber[cnt] != SectionNumber[cnt]) break;
         if (cnt < HeadNumber+1) continue;
      }
      else
      if (SynopsisLevel == -2)
      {
         if (cdptr->SectionNumber[0] <= SectionNumber[0]) continue;
         if (cdptr->HeadNumber != HeadNumber+2) continue;
         for (cnt = 1; cnt < HeadNumber+2; cnt++)
            if (cdptr->SectionNumber[cnt] != SectionNumber[cnt]) break;
         if (cnt < HeadNumber+2) continue;
      }

      if (ListLength-- <= 0) break;

      if (TagPtr)
      {
         /* synopsize matching $$TAG$$ */
         HitTag = FALSE;
         TagParse (NULL);
         tptr = csptr->ContentPtr->TextPtr;
         while (cptr = TagParse(&tptr))
            if (strsame (cptr, TagPtr, -1)) break;
         if (!cptr) continue;
      }

      /* allocate sufficient to contain the entire section! */
      if (cdptr->TextLength > ExtractSize)
      {
         if (ExtractPtr)
         {
            CgiLibVeeMemFree (ExtractPtr);
            if (vaxc$errno) ErrorExit (vaxc$errno, FI_LI);
         }
         ExtractSize = cdptr->TextLength;
         ExtractPtr = CgiLibVeeMemCalloc (ExtractSize);
         if (!ExtractPtr) ErrorExit (vaxc$errno, FI_LI);
      }
      strcpy (ExtractPtr, csptr->ExtractPtr);

      if (CloudPtr)
      {
         /* cloud is essentially searching the entire section */
         SearchStripMarkup (ExtractPtr);
         if (!SearchKeyWord (ExtractPtr, CloudPtr)) continue;
      }

      /*************/
      /* synopsize */
      /*************/

      ItemCount++;

      if (!CloudPtr)
      {
         cptr = ExtractPtr;
         while (*(cptr = TemplateParseFind (cptr, "$$SYNOPSIS$")))
         {
            if (strsame (cptr, "$$SYNOPSIS$=", 12))
            {
               strcpy (ExtractPtr, cptr+12);
               break;
            }
            if (strsame (cptr, "$$SYNOPSIS$text=", 16))
            {
               strcpy (ExtractPtr, cptr+16);
               break;
            }
            cptr += 10;
         }
         SearchStripMarkup (ExtractPtr);
      }

      if (ExtractLength)
      {
         /* pick up any trailing parts of 'words' */
         zptr = ExtractPtr + ExtractLength;
         for (cptr = ExtractPtr; *cptr && cptr < zptr; cptr++);
         zptr = ExtractPtr + ExtractSize;
         while (*cptr && (isalnum(*cptr) || *cptr == '-') && cptr < zptr)
            cptr++;
         *cptr = '\0';
     }
     else
         *ExtractPtr = '\0';

      /* deentify to convert entities into UTF-8 then rentify twice below! */
      if (QueryRss) ContentDeEntify (ExtractPtr, TRUE);

      if (SynopsisLevel == -2)
      {
         /* show the parent section title along with the synopsised section */
         for (cptr = cdptr->SectionName; *cptr; cptr++);
         while (cptr > cdptr->SectionName && *cptr != '/') cptr--;
         if (cptr > cdptr->SectionName) cptr--;
         while (cptr > cdptr->SectionName && *cptr != '/') cptr--;
         if (cptr > cdptr->SectionName) cptr++;
         zptr = (sptr = SynopsisTitle) + sizeof(SynopsisTitle)-1;
         while (*cptr && *cptr != '/' && sptr < zptr) *sptr++ = *cptr++;
         if (*cptr)
         {
            if (sptr < zptr) *sptr++ = ' ';
            if (sptr < zptr) *sptr++ = *cptr++;
            if (sptr < zptr) *sptr++ = ' ';
         }
         while (*cptr && *cptr != '/' && sptr < zptr) *sptr++ = *cptr++;
         *sptr = '\0';
         TitlePtr = SynopsisTitle;
      }
      else
         TitlePtr = cdptr->SectionTitle;

      if (QueryRss)
      {
         fprintf (stdout,
"   <item>\n\
    <title>%s</title>\n\
    <pubDate>%s %s</pubDate>\n\
    <description>%s%s</description>\n\
    <link>%s%s</link>\n\
    <guid>%s%s</guid>\n\
   </item>\n",
                  RssEscape(TitlePtr),
                  TimeString("%a, %d %b %Y %H:%M:%S",csptr->SectionLastUpdated),
                  TimeZoneString,
                  RssEscape(ExtractPtr),
                  *cptr ? "..." : "",
                  LinkSchemeHost,
                  RssEscape(ContentEncodeName("/",cdptr->SectionName)),
                  LinkSchemeHost,
                  RssEscape(ContentEncodeName("/",cdptr->SectionName)));
      }
      else
      if (SynopsisSubsArg)
      {
         if (SinceSeconds)
            fprintf (stdout,
"<tr><td class=\"vwcms_synopsis_entry\">\
<span class=\"vwcms_synopsis_title\">%s</span>\
<span class=\"vwcms_synopsis_datetime%s\">%s&nbsp;&nbsp;(%s)</span>\
<span class=\"vwcms_synopsis_extract%s%s\">%s\
<span class=\"vwcms_synopsis_read%s\">%s",
                     HtmlEscape(TitlePtr),
                     NarrowClass,
                     TimeString(NULL,csptr->SectionLastUpdated),
                     AgoWas(TimeString("VMS20",csptr->SectionLastUpdated)),
                     NarrowClass,
                     *ExtractPtr ? "" : "_0",
                     HtmlEscape(ExtractPtr),
                     *ExtractPtr ? "" : NarrowClass,
                     *cptr ? "..." : "");
         else
            fprintf (stdout,
"<tr><td class=\"vwcms_synopsis_entry\">\
<span class=\"vwcms_synopsis_title\">%s</span>\
<span class=\"vwcms_synopsis_extract%s%s\">%s\
<span class=\"vwcms_synopsis_read%s\">%s",
                     HtmlEscape(TitlePtr),
                     NarrowClass,
                     *ExtractPtr ? "" : "_0",
                     HtmlEscape(ExtractPtr),
                     *ExtractPtr ? "" : NarrowClass,
                     *cptr ? "..." : "");
      }
      else
      {
         fprintf (stdout,
"<tr><td class=\"vwcms_blog_list_entry\">\
<span class=\"vwcms_blog_list_title\">%s</span>\
<span class=\"vwcms_blog_list_datetime%s\">%s&nbsp;&nbsp;(%s)</span>\
<span class=\"vwcms_blog_list_extract%s%s\">%s\
<span class=\"vwcms_blog_list_read%s\">%s",
                  HtmlEscape(cdptr->SectionTitle),
                  NarrowClass,
                  TimeString(NULL,csptr->TimeStampSeconds),
                  AgoWas(TimeString("VMS20",csptr->TimeStampSeconds)),
                  NarrowClass,
                  *ExtractPtr ? "" : "_0",
                  HtmlEscape(ExtractPtr),
                  *ExtractPtr ? "" : NarrowClass,
                  *cptr ? "..." : "");
      }

      if (!QueryRss)
      {
         SectionNamePtr = cdptr->SectionName;
         if (cptr = cdptr->BlogPtr)
         {
            while (*cptr && *(USHORTPTR)cptr != '$$')
            {
               if (strsame (cptr, "goto=", 5))
               {
                  TemplateParseParam (cptr+5, SynopsisGoto,
                                      sizeof(SynopsisGoto));
                  SectionNamePtr = SynopsisGoto;
                  break;
               }
               cptr = TemplateParseSkipParam (cptr, NULL);
            }
         }

         if (cptr = IsAnImage (ReadPtr))
            fprintf (stdout,
"<a%s href=\"%s\"><img class=\"vwcms_nav_img\" alt=\"[IMAGE]\" %s /></a>",
                     ManageLeaveNow(),
                     ContentEncodeName("/",SectionNamePtr),
                     cptr);
         else
            fprintf (stdout,
"[<a%s href=\"%s\">%s</a>]",
                     ManageLeaveNow(),
                     ContentEncodeName("/",SectionNamePtr),
                     ReadPtr);

         fputs ("</span></span>\n</td></tr>\n", stdout);
      }
   }

   if (ExtractPtr)
   {
      CgiLibVeeMemFree (ExtractPtr);
      if (vaxc$errno) ErrorExit (vaxc$errno, FI_LI);
   }

   /************/
   /* end list */
   /************/

   if (QueryRss)
   {
      if (!ItemCount)
         fprintf (stdout,
"   <item>\n\
    <title>%s</title>\n\
    <pubDate>%s %s</pubDate>\n\
   </item>\n",
                  SynopsisEmpty[0] ? SynopsisEmpty : CONFIG_LANG(LangEmpty),
                  TimeString("%a, %d %b %Y %H:%M:%S",0),
                  TimeZoneString);

      fputs (" </channel>\n</rss>\n", stdout);
   }
   else
   if (SynopsisSubsArg)
   {
      if (!ItemCount)
         fprintf (stdout,
"<tr><td class=\"vwcms_synopsis_entry\"><div class=\"vwcms_no_content\">\
%s</div></td></tr>\n",
                  SynopsisEmpty[0] ? SynopsisEmpty : CONFIG_LANG(LangEmpty));

      /* also returns from $$SYNOPSIS$.$$ above */
      fputs ("</table>\n", stdout);
   }
   else
   {
      if (!ItemCount)
         fprintf (stdout,
"<tr><td class=\"vwcms_blog_list_entry\"><div class=\"vwcms_no_content\">\
%s</div></td></tr>\n",
                  SynopsisEmpty[0] ? SynopsisEmpty : CONFIG_LANG(LangEmpty));

      /* also returns from $$SYNOPSIS$.$$ above */
      fputs ("</table>\n", stdout);
   }
}

/*****************************************************************************/
